﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace TPCWare.CDays14.Twitter.Service.Model
{
    internal static class Utils
    {
        private static string unescapedChars = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-_.~";

        internal static char DoubleQuoteChar = '"';

        // Encode according to RFC 3986 Section 2.1 (see https://dev.twitter.com/docs/auth/percent-encoding-parameters)
        internal static string PercentEncode(string s)
        {
            StringBuilder result = new StringBuilder();

            foreach (char c in s)
            {
                if (unescapedChars.IndexOf(c) != -1)
                {
                    result.Append(c);
                }
                else
                {
                    result.Append('%' + String.Format("{0:X2}", (int)c));
                }
            }

            return result.ToString();
        }

        internal static byte[] StringToAscii(string s)
        {
            byte[] retval = new byte[s.Length];
            for (int ix = 0; ix < s.Length; ++ix)
            {
                char ch = s[ix];
                if (ch <= 0x7f)
                {
                    retval[ix] = (byte)ch;
                }
                else
                {
                    // This should never happen because the input string should allways be a percent encoded one.
                    // Instead of replacing the offending char with retval[ix] = (byte)'?';
                    // we prefere to raise an error, because in this case, the twitter request will surely fail.
                    throw new Exception(string.Format("StringToAscii Exception: non ASCII char in string: {0}", s));
                }
            }
            return retval;
        }

        internal static string AsciiToString(byte[] bytes)
        {
            StringBuilder sb = new StringBuilder(bytes.Length);
            foreach (byte b in bytes)
            {
                if (b <= 0x7f)
                {
                    sb.Append((char)b);
                }
                else
                {
                    // This should never happen because the byte array should contain only ASCII chars.
                    // Instead of replacing the offending char with '?', we prefere to raise an error because
                    // in this case, the twitter request will surely fail.
                    throw new Exception("AsciiToString Exception: non ASCII char in input byte array");
                }
            }
            return sb.ToString();
        }

    }
}
