﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.Http;
using System.Text;
using System.Security.Cryptography;
using System.Threading.Tasks;
using System.Net.Http.Headers;
using System.Net;
using TPCWare.CDays14.Twitter.Service.Model;

namespace TPCWare.CDays14.Twitter.Service
{
    public class SpreakerTwitterService
    {
        private HttpClient httpClient;

        private string oauthConsumerKey;
        private string oauthConsumerSecret;
        private string oauthCallbackUrl;

        public SpreakerTwitterService(string oauthConsumerKey, string oauthConsumerSecret, string oauthCallbackUrl)
        {
            httpClient = new HttpClient();
            httpClient.BaseAddress = new Uri("https://api.twitter.com");

            this.oauthConsumerKey = oauthConsumerKey;
            this.oauthConsumerSecret = oauthConsumerSecret;
            this.oauthCallbackUrl = oauthCallbackUrl;
        }

        public async Task<RequestToken> GetRequestToken()
        {
            RequestToken requestToken = null;

            string oauthNonce = OAuthNonce.Value();
            string oauthTimestamp = Convert.ToInt64((DateTime.UtcNow - new DateTime(1970, 1, 1)).TotalSeconds).ToString();

            string httpMethod = "POST";
            string baseUrl = "https://api.twitter.com/oauth/request_token";

            var optionalParameters = new Dictionary<string, string>()
            {
                { "oauth_callback", oauthCallbackUrl }
            };

            var signature = new Signature(httpMethod, baseUrl, oauthNonce, oauthTimestamp, oauthConsumerKey, oauthConsumerSecret, null, null, optionalParameters);
            string requestSignature = signature.Compute();

            var requestParameters = new Dictionary<string, string>()
            {
                { "oauth_callback", oauthCallbackUrl },
                { "oauth_consumer_key", oauthConsumerKey },
                { "oauth_nonce", oauthNonce },
                { "oauth_signature", requestSignature },
                { "oauth_signature_method", "HMAC-SHA1" },
                { "oauth_timestamp", oauthTimestamp },
                { "oauth_version", "1.0" }
            };

            string authorizationHeaderValue = OAuthAuthorizationHeader.Value(requestParameters);

            try
            {
                // Fetch request token
                var request = new HttpRequestMessage();
                request.RequestUri = new Uri("https://api.twitter.com/oauth/request_token");
                request.Method = HttpMethod.Post;
                request.Headers.Accept.Add(new MediaTypeWithQualityHeaderValue("*/*"));
                request.Headers.Add("Authorization", authorizationHeaderValue);

                var response = await httpClient.SendAsync(request);

                if (response.StatusCode == HttpStatusCode.OK)
                {
                    // Get data from result
                    string body = await response.Content.ReadAsStringAsync();

                    // Fill the request token properties
                    requestToken = new RequestToken(body);
                }
            }
            catch(Exception e)
            {
                // No need to raise an error. The request token will remain null,
                // indicating that request has failed.
            }

            return requestToken;
        }

        public async Task<AccessToken> GetUserAccessToken(RedirectToken redirectToken)
        {
            AccessToken accessToken = null;

            string oauthNonce = OAuthNonce.Value();
            string oauthTimestamp = Convert.ToInt64((DateTime.UtcNow - new DateTime(1970, 1, 1)).TotalSeconds).ToString();

            string httpMethod = "POST";
            string baseUrl = "https://api.twitter.com/oauth/access_token";

            var optionalParameters = new Dictionary<string, string>()
            {
                { "oauth_verifier", redirectToken.OauthVerifier }
            };

            var signature = new Signature(httpMethod, baseUrl, oauthNonce, oauthTimestamp, oauthConsumerKey, oauthConsumerSecret, redirectToken.OauthToken, null, optionalParameters);
            string requestSignature = signature.Compute();

            var requestParameters = new Dictionary<string, string>()
            {
                { "oauth_consumer_key", oauthConsumerKey },
                { "oauth_nonce", oauthNonce },
                { "oauth_signature", requestSignature },
                { "oauth_signature_method", "HMAC-SHA1" },
                { "oauth_timestamp", oauthTimestamp },
                { "oauth_token", redirectToken.OauthToken },
                { "oauth_version", "1.0" }
            };

            string authorizationHeaderValue = OAuthAuthorizationHeader.Value(requestParameters);

            try
            {
                // Fetch access token
                var request = new HttpRequestMessage();
                request.RequestUri = new Uri("https://api.twitter.com/oauth/access_token");
                request.Method = HttpMethod.Post;
                request.Headers.Accept.Add(new MediaTypeWithQualityHeaderValue("*/*"));
                request.Headers.Add("Authorization", authorizationHeaderValue);
                request.Content = new FormUrlEncodedContent(new[] 
                {
                    new KeyValuePair<string, string>("oauth_verifier", redirectToken.OauthVerifier)
                });
                
                var response = await httpClient.SendAsync(request);

                if (response.StatusCode == HttpStatusCode.OK)
                {
                    // Get data from result
                    string body = await response.Content.ReadAsStringAsync();

                    // Fill the access token properties
                    accessToken = new AccessToken(body);
                }
            }
            catch(Exception e)
            {
                // No need to raise an error. The access token will remain null,
                // indicating that request has failed.
            }

            return accessToken;
        }

    }
}
